import { Hono } from 'hono'
import { bearerAuth } from 'hono/bearer-auth'
import { zValidator } from '@hono/zod-validator'
import { z } from 'zod'
import { EmailService } from '../services/email'
import { getEmailTemplate } from '../templates'
import { Language } from '../types'
import { Attachment } from 'nodemailer/lib/mailer'

const emailRouter = new Hono()
const emailService = new EmailService()

const sendEmailSchema = z.object({
  to: z.string().email(),
  templateName: z.string(),
  language: z.enum(['en', 'de']),
  data: z.record(z.any())
})

async function generateBarcodeDataURL(data) {
  return `${process.env.DOCUMENT_SERVER_URL}/v1/barcodes/code128/${data}`
}

emailRouter.post('/', bearerAuth({ token: process.env.AUTHKEY }), zValidator('json', sendEmailSchema), async (c) => {
  let { to, templateName, language, data } = c.req.valid('json')
  const attachments: Attachment[] = []
  try {
    const template = getEmailTemplate(templateName, language as Language)
    if (templateName === "welcome") {
      if (data.name && data.link && data.barcode_content) {
        const dataURL = await generateBarcodeDataURL(data.barcode_content);
        data.barcode_url = dataURL;
      } else {
        return c.json({ success: false, error: "required params 'data.name', 'data.link', 'data.barcode_content' not provided" }, 406)
      }
      const attachment: Attachment = {
        filename: 'invite.ics',
        content: `BEGIN:VCALENDAR\nVERSION:2.0\nBEGIN:VEVENT\nSUMMARY:Lauf für Kaya! 2025\nDTSTART:20250523T110000Z\nDTEND:20250523T160000Z\nDTSTAMP:20250318T230306Z\nUID:lfk2025-xhJP3h\nDESCRIPTION:Der Lauf für Kaya! 2025 findet am 23.05.2025 auf dem Sportplatz des Gymnasium Herzogenaurach statt - Bürgerlauf von 14 bis 18 Uhr - Firmenlauf von 13:30 bis 18 Uhr.\nLOCATION:Sportplatz Gymnasium Herzogenaurach\nORGANIZER:info@lauf-fuer-kaya.de\nSTATUS:CONFIRMED\nPRIORITY:5\nEND:VEVENT\nEND:VCALENDAR`,
        contentType: 'text/calendar; method=REQUEST',
      }
      attachments.push(attachment)
    }
    if (templateName === "password-reset") {
      if (data.token) {
        console.log(data);
        data.reset_link = `${process.env.FRONTEND_URL}/reset/${(Buffer.from(data.token)).toString("base64")}`
      } else {
        return c.json({ success: false, error: "'data.token' not provided" }, 406)
      }
    }
    data.event_date = process.env.EVENT_DATE
    data.event_name = process.env.EVENT_NAME
    await emailService.sendEmail({
      to,
      attachments,
      subject: template.subject(data),
      html: template.html(data),
      text: template.text(data)
    })

    return c.json({ success: true })
  } catch (error) {
    return c.json({ success: false, error: (error as Error).message }, 400)
  }
})

// Add queue status endpoint
emailRouter.get('/status', bearerAuth({ token: process.env.AUTHKEY }), async (c) => {
  try {
    const status = await emailService.getQueueStatus()
    return c.json(status)
  } catch (error) {
    return c.json({ error: (error as Error).message }, 500)
  }
})

export { emailRouter }